import 'package:flutter/cupertino.dart';

class NUIResponsiveContainerUtil{
  NUIResponsiveContainerUtil._builder({this.optimumHeight, this.optimumWidth}){
    assert(optimumWidth != null || optimumHeight != null);
  }
  final double optimumWidth;
  final double optimumHeight;

  double _currentWidth;
  double _currentHeight;

  double get currentWidth{
    return _currentWidth;
  }

  double get currentHeight{
    return _currentHeight;
  }

  double scaledOptimumWidth(double value){
    final initialValue = value;
    final scaleRatio = initialValue/optimumWidth;
    final scaledValue = _currentWidth * scaleRatio;
    return scaledValue;
  }

  double get widthScale{
    final scaleRatio = _currentWidth/optimumWidth;
    return scaleRatio < 1 ? scaleRatio : 1.0;
  }

  double get heightScale{
    final scaleRatio = _currentHeight/optimumHeight;
    return scaleRatio < 1 ? scaleRatio : 1.0;
  }

  double scaledOptimumHeight(double value){
    final initialValue = value;
    final scaleRatio = initialValue/optimumHeight;
    final scaledValue = _currentHeight * scaleRatio;
    return scaledValue;
  }
}

typedef NUIResponsiveContainerBuilder = Widget Function(BuildContext context, NUIResponsiveContainerUtil util);

class NUIResponsiveContainer extends StatefulWidget {
  NUIResponsiveContainer({@required this.builder, this.optimumHeight, this.optimumWidth, this.fitted: false, Key key}): super(key: key){
    assert(optimumWidth != null || optimumHeight != null);
    assert(builder != null);
  }
  final double optimumWidth;
  final double optimumHeight;
  final bool fitted;
  final NUIResponsiveContainerBuilder builder;
  @override
  _NUIResponsiveContainerState createState() => _NUIResponsiveContainerState();
}

class _NUIResponsiveContainerState extends State<NUIResponsiveContainer> {
  NUIResponsiveContainerUtil _util;

  @override
  void initState(){
    super.initState();
    _util = NUIResponsiveContainerUtil._builder(optimumWidth: widget.optimumWidth, optimumHeight: widget.optimumHeight);
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
        builder: (context, box){
          final maxWidth = box.maxWidth;
          final maxHeight = box.maxHeight;

          _util._currentHeight = maxHeight;
          _util._currentWidth = maxWidth;

          if(widget.fitted == true){
            return FittedBox(child: SizedBox(width: widget.optimumWidth, height: widget.optimumHeight, child: widget.builder(context, _util)));
          }
          else {
            return widget.builder(context, _util);
          }
        }
    );
  }
}
