import 'package:flutter/cupertino.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/string/nui_string_util.dart';

abstract class NUITooltipOverlayMixin {
  bool _overlayAdded = false;
  String _overlayingType;
  OverlayEntry _overlayEntry;

  bool get showingOverlay{
    return _overlayAdded;
  }

  Widget getTooltipOverlay(BuildContext context, {Map<String, dynamic> params});

  void renderTooltip(BuildContext context, String type, {Map<String, dynamic> params}){
    bool isSameType = match(type, _overlayingType);
    if(isSameType && _overlayAdded) return; //Already showing overlay
    // if(_overlayEntry?.mounted == true){
      try {
        _overlayEntry.remove();
      }catch(e){}
    // }

    final overlayChild = getTooltipOverlay(context, params: params);
    _overlayingType = type;
    _overlayAdded = true;
    _overlayEntry = OverlayEntry(builder: (context){
      return overlayChild;
    });

    _overlayAdded = true;
    NUIAsync.delay(Duration(milliseconds: 10), () {
      if(_overlayEntry.mounted) return;
      try {
        Overlay.of(context).insert(_overlayEntry);
      }catch(e){}
    });
  }

  void cancelTooltipOverlay(){
    if(!_overlayAdded) return;

    try{
      _overlayingType = null;
      _overlayAdded = false;
      _overlayEntry?.remove();
    }catch(e){}
  }

}
