import 'package:flutter/material.dart';

//Contributed by Leslie Ling

class NUIVerticalExpandable extends StatefulWidget {

  final Widget child;
  final bool expand;
  final Duration duration;
  NUIVerticalExpandable({this.expand = false, @required this.child, @required this.duration});

  @override
  _NUIVerticalExpandableState createState() => _NUIVerticalExpandableState();
}

class _NUIVerticalExpandableState extends State<NUIVerticalExpandable> with SingleTickerProviderStateMixin {
  AnimationController expandController;
  Animation<double> animation;

  @override
  void initState() {
    super.initState();
    _prepareAnimations();

    if(widget.expand) {
      expandController.forward();
    }
    else {
      expandController.reverse();
    }
  }

  void _prepareAnimations() {
    expandController = AnimationController(
        vsync: this,
        duration: widget.duration ?? Duration(milliseconds: 500)
    );
    Animation curve = CurvedAnimation(
      parent: expandController,
      curve: Curves.fastOutSlowIn,
    );
    animation = Tween(begin: 0.0, end: 1.0).animate(curve)
      ..addListener(() {
        setState(() {});
      });
  }

  @override
  void didUpdateWidget(NUIVerticalExpandable oldWidget) {
    super.didUpdateWidget(oldWidget);
    if(widget.expand) {
      expandController.forward();
    }
    else {
      expandController.reverse();
    }
  }

  @override
  void dispose() {
    expandController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedOpacity(
      opacity: (widget.expand) ? 1.0 : 0.0,
      duration: widget.duration ?? Duration(milliseconds: 500),
      child: SizeTransition(
          axisAlignment: 1.0,
          sizeFactor: animation,
          child: widget.child
      ),
    );
  }
}