import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/nui_core.dart';

class NUIPageViewIndicator extends StatefulWidget {
  static const double _defaultSize = 4.0;
  static const double _defaultSelectedSize = 8.0;
  static const double _defaultSpacing = 8.0;
  static Color _defaultIndicatorColor = Colors.white.withOpacity(0.6);
  static Color _defaultSelectedIndicatorColor = Colors.white;

  final NUIBlocData<int> currentPageNotifier;
  final int itemCount;
  final int maximumCount;
  final ValueChanged<int> onPageSelected;
  final BorderRadius borderRadius;
  final Color indicatorColor;
  final Color selectedIndicatorColor;
  final double size;
  final int currentPosition;
  final double selectedSize;
  final double spacing;
  final bool isCircle;

  NUIPageViewIndicator({
    Key key,
    @required this.currentPageNotifier,
    @required this.itemCount,
    this.onPageSelected,
    this.maximumCount,
    this.currentPosition: 0,
    this.size = _defaultSize,
    this.spacing = _defaultSpacing,
    this.borderRadius,
    this.isCircle: true,
    Color color,
    Color selectedColor,
    this.selectedSize = _defaultSelectedSize,
  })  : this.indicatorColor = color ??
      ((selectedColor?.withAlpha(150)) ?? _defaultIndicatorColor),
        this.selectedIndicatorColor = selectedColor ?? _defaultSelectedIndicatorColor,
        super(key: key);

  @override
  NUIPageViewIndicatorState createState() {
    return new NUIPageViewIndicatorState();
  }
}

class NUIPageViewIndicatorState extends State<NUIPageViewIndicator> with TickerProviderStateMixin{
  int _currentPageIndex = 0;
  List<int> _currentIndexQueue = [];

  @override
  void initState() {
    super.initState();
    _currentPageIndex = widget.currentPosition ?? 0;
  }

  // void _fillCurrentQueue(){
  //   _currentIndexQueue.clear();
  //   if(widget.itemCount <= widget.maximumCount){
  //     for(int i = 0; i< widget.itemCount; i++){
  //       _currentIndexQueue.add(i);
  //     }
  //     return;
  //   }
  //
  //   if(_currentPageIndex == 0){
  //     for(int i = 0; i< widget.maximumCount; i++){
  //       _currentIndexQueue.add(i);
  //     }
  //   }
  //   else if(_currentPageIndex == widget.itemCount - 1){
  //     for(int i = widget.itemCount - widget.maximumCount; i< widget.maximumCount; i++){
  //       _currentIndexQueue.add(i);
  //     }
  //   }
  //   else{
  //
  //   }
  // }

  @override
  void dispose() {
    super.dispose();
  }

  // int _getCurrentStartingIndex(){
  //   if(widget.maximumCount == null) return 0;
  //   if(widget.itemCount <= widget.maximumCount) return 0;
  //
  //
  //
  // }
  //
  // int _getCurrentEndingIndex(){
  //   if(widget.maximumCount == null) return widget.itemCount - 1;
  //   if(widget.itemCount <= widget.maximumCount) return widget.itemCount - 1;
  // }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: widget.selectedSize ?? NUIPageViewIndicator._defaultSelectedSize,
      child: NUIBlocStream<int>(
        bloc: widget.currentPageNotifier,
        initial: _currentPageIndex,
        stream: (context, status, curIndex) => NUIRow(
            divider: VerticalDivider(width: widget.spacing ?? NUIPageViewIndicator._defaultSpacing, color: NUIColors.NUITransparent),
            mainAxisSize: MainAxisSize.min,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: List<Widget>.generate(widget.itemCount, (int index) {
              double size = widget.size ?? NUIPageViewIndicator._defaultSize;
              double selectedSize = widget.selectedSize ?? NUIPageViewIndicator._defaultSelectedSize;
              Color color = widget.indicatorColor ?? NUIPageViewIndicator._defaultIndicatorColor;
              Color selectedColor = widget.selectedIndicatorColor ?? NUIPageViewIndicator._defaultSelectedIndicatorColor;

              _currentPageIndex = curIndex;
              bool selected = isSelected(index);

              return AnimatedSize(
                vsync: this,
                duration: Duration(milliseconds: 250),
                child: widget.isCircle != true ? Container(
                  width: selected ? selectedSize : size,
                  height: selected ? selectedSize : size,
                  decoration: NUIBoxDecoration(
                      color: selected ? selectedColor : color,
                      borderRadius: widget.borderRadius ?? BorderRadius.circular((selected ? selectedSize : size) / 2)
                  ),
                ) : CircleAvatar(
                  backgroundColor: selected ? selectedColor : color,
                  radius: (selected ? selectedSize : size) / 2,
                ),
              );
            })
        ),
      ),
    );
  }

  bool isSelected(int dotIndex) {
    return dotIndex == _currentPageIndex;
  }

  _handlePageIndex() {
    // logNUI("PageViewIndicator", "New index : ${widget.currentPageNotifier.value}");
    setState(_readCurrentPageIndex);
  }

  _readCurrentPageIndex() {
    // _currentPageIndex = widget.currentPageNotifier.value;
  }
}
